@@ -221,6 +221,7 @@ public:
 
 			Firebird::ClumpletWriter dpb(Firebird::ClumpletReader::dpbList, MAX_DPB_SIZE);
 			dpb.insertByte(isc_dpb_sec_attach, TRUE);
+			dpb.insertString(isc_dpb_config, EMBEDDED_PROVIDERS, fb_strlen(EMBEDDED_PROVIDERS));
 
 			unsigned int authBlockSize;
 			const unsigned char* authBlock = logonInfo->authBlock(&authBlockSize);
@@ -675,7 +676,7 @@ private:
 	Firebird::RefPtr<Firebird::IFirebirdConf> config;
 	Firebird::RefPtr<Firebird::IAttachment> att;
 	Firebird::RefPtr<Firebird::ITransaction> tra;
-	RemotePassword server;
+	RemotePasswordImpl<Firebird::Sha1> server;
 	int upCount, delCount;
 
 	bool checkCount(Firebird::CheckStatusWrapper* status, int* count, UCHAR item)
