@@ -323,17 +323,12 @@ bool Parser::yylexSkipSpaces()
 		if (lex.ptr >= lex.end)
 			return false;
 
-		c = *lex.ptr++;
+		if (yylexSkipEol())
+			continue;
 
 		// Process comments
 
-		if (c == '\n')
-		{
-			lex.lines++;
-			lex.line_start = lex.ptr;
-			continue;
-		}
-
+		c = *lex.ptr++;
 		if (c == '-' && lex.ptr < lex.end && *lex.ptr == '-')
 		{
 			// single-line
@@ -341,12 +336,9 @@ bool Parser::yylexSkipSpaces()
 			lex.ptr++;
 			while (lex.ptr < lex.end)
 			{
-				if ((c = *lex.ptr++) == '\n')
-				{
-					lex.lines++;
-					lex.line_start = lex.ptr; // + 1; // CVC: +1 left out.
+				if (yylexSkipEol())
 					break;
-				}
+				lex.ptr++;
 			}
 			if (lex.ptr >= lex.end)
 				return false;
@@ -361,17 +353,14 @@ bool Parser::yylexSkipSpaces()
 			lex.ptr++;
 			while (lex.ptr < lex.end)
 			{
+				if (yylexSkipEol())
+					continue;
+
 				if ((c = *lex.ptr++) == '*')
 				{
 					if (*lex.ptr == '/')
 						break;
 				}
-				if (c == '\n')
-				{
-					lex.lines++;
-					lex.line_start = lex.ptr; // + 1; // CVC: +1 left out.
-
-				}
 			}
 			if (lex.ptr >= lex.end)
 			{
@@ -395,6 +384,34 @@ bool Parser::yylexSkipSpaces()
 }
 
 
+bool Parser::yylexSkipEol()
+{
+	bool eol = false;
+	const TEXT c = *lex.ptr;
+	if (c == '\r')
+	{
+		lex.ptr++;
+		if (lex.ptr < lex.end && *lex.ptr == '\n')
+			lex.ptr++;
+
+		eol = true;
+	}
+	else if (c == '\n')
+	{
+		lex.ptr++;
+		eol = true;
+	}
+
+	if (eol)
+	{
+		lex.lines++;
+		lex.line_start = lex.ptr; // + 1; // CVC: +1 left out.
+	}
+
+	return eol;
+}
+
+
 int Parser::yylexAux()
 {
 	thread_db* tdbb = JRD_get_thread_data();
