@@ -477,7 +477,7 @@ public:
 	bool dataFlag, downFlag;
 };
 
-typedef GenericMap<Pair<Left<NoCaseString, Cache*> > > CacheTree;
+typedef GenericMap<Pair<Left<NoCaseString, AutoPtr<Cache> > > > CacheTree;
 
 InitInstance<CacheTree> tree;
 GlobalPtr<Mutex> treeMutex;
@@ -487,8 +487,8 @@ void setupIpc();
 Cache* locate(const NoCaseString& target)
 {
 	fb_assert(treeMutex->locked());
-	Cache* c;
-	return tree().get(target, c) ? c : NULL;
+	AutoPtr<Cache> *c = tree().get(target);
+	return c ? c->get() : NULL;
 }
 
 Cache* locate(const NoCaseString& alias, const NoCaseString& target)
@@ -508,7 +508,7 @@ Cache* locate(const NoCaseString& alias, const NoCaseString& target)
 class Found
 {
 public:
-	enum What {FND_NOTHING, FND_SEC, FND_DB};
+	enum What {FND_NOTHING, FND_PLUG, FND_SEC, FND_DB};
 
 	Found()
 		: found(FND_NOTHING)
@@ -516,8 +516,13 @@ public:
 
 	void set(What find, const AuthReader::Info& val)
 	{
+		fb_assert(find != FND_NOTHING);
+
+		if (val.plugin.hasData())
+			find = FND_PLUG;
 		if (find == found && value != val.name)
 			Arg::Gds(isc_map_undefined).raise();
+
 		if (find > found)
 		{
 			found = find;
@@ -686,8 +691,8 @@ public:
 				if (!ISC_check_process_existence(p->id))
 				{
 					p->flags &= ~MappingHeader::FLAG_ACTIVE;
-					sharedMemory->eventFini(&sMem->process[process].notifyEvent);
-					sharedMemory->eventFini(&sMem->process[process].callbackEvent);
+					sharedMemory->eventFini(&p->notifyEvent);
+					sharedMemory->eventFini(&p->callbackEvent);
 					break;
 				}
 			}
@@ -725,7 +730,7 @@ public:
 		{
 			if (!(sMem->process[process].flags & MappingHeader::FLAG_ACTIVE))
 				break;
-			if (!ISC_check_process_existence(processId))
+			if (!ISC_check_process_existence(sMem->process[process].id))
 			{
 				sharedMemory->eventFini(&sMem->process[process].notifyEvent);
 				sharedMemory->eventFini(&sMem->process[process].callbackEvent);
@@ -886,7 +891,7 @@ void setupIpc()
 	mappingIpc->setup();
 }
 
-class DbHandle : public AutoPtr<IAttachment, SimpleRelease<IAttachment> >
+class DbHandle : public AutoPtr<IAttachment, SimpleRelease>
 {
 public:
 	DbHandle()
@@ -917,6 +922,7 @@ public:
 		ClumpletWriter embeddedSysdba(ClumpletWriter::Tagged, 1024, isc_dpb_version1);
 		embeddedSysdba.insertString(isc_dpb_user_name, SYSDBA_USER_NAME, fb_strlen(SYSDBA_USER_NAME));
 		embeddedSysdba.insertByte(isc_dpb_sec_attach, TRUE);
+		embeddedSysdba.insertString(isc_dpb_config, EMBEDDED_PROVIDERS, fb_strlen(EMBEDDED_PROVIDERS));
 		embeddedSysdba.insertByte(isc_dpb_map_attach, TRUE);
 		embeddedSysdba.insertByte(isc_dpb_no_db_triggers, TRUE);
 
@@ -982,15 +988,21 @@ bool mapUser(string& name, string& trusted_role, Firebird::string* auth_method,
 	// Create new writer
 	AuthWriter newBlock;
 
-	// detect presence of this databases mapping in authBlock
+	// detect presence of non-plugin databases mapping in authBlock
 	// in that case mapUser was already invoked for it
 	unsigned flags = db ? 0 : FLAG_DB;
 	for (AuthReader rdr(authBlock); rdr.getInfo(info); rdr.moveNext())
 	{
-		if (db && info.secDb == db)
-			flags |= FLAG_DB;
-		if (info.secDb == securityDb)
-			flags |= FLAG_SEC;
+		MAP_DEBUG(fprintf(stderr, "info.plugin=%s info.secDb=%s db=%s securityDb=%s\n",
+			info.plugin.c_str(), info.secDb.c_str(), db ? db : "NULL", securityDb));
+
+		if (info.plugin.isEmpty())
+		{
+			if (db && info.secDb == db)
+				flags |= FLAG_DB;
+			if (info.secDb == securityDb)
+				flags |= FLAG_SEC;
+		}
 	}
 
 	// Perform lock & map only when needed
@@ -1213,6 +1225,7 @@ RecordBuffer* MappingList::getList(thread_db* tdbb, jrd_rel* relation)
 		embeddedSysdba.insertString(isc_dpb_user_name, SYSDBA_USER_NAME,
 			fb_strlen(SYSDBA_USER_NAME));
 		embeddedSysdba.insertByte(isc_dpb_sec_attach, TRUE);
+		embeddedSysdba.insertString(isc_dpb_config, EMBEDDED_PROVIDERS, fb_strlen(EMBEDDED_PROVIDERS));
 		embeddedSysdba.insertByte(isc_dpb_no_db_triggers, TRUE);
 
 		const char* dbName = tdbb->getDatabase()->dbb_config->getSecurityDatabase();
