@@ -149,7 +149,7 @@ MonitoringData::~MonitoringData()
 {
 	Guard guard(this);
 
-	if (shared_memory->getHeader()->used == sizeof(Header))
+	if (shared_memory->getHeader()->used == alignOffset(sizeof(Header)))
 		shared_memory->removeMapFile();
 }
 
@@ -193,7 +193,6 @@ void MonitoringData::read(const char* user_name, TempSpace& temp)
 
 		if (!user_name || !strcmp(element->userName, user_name))
 		{
-			fb_assert(shared_memory->getHeader()->used >= offset + length);
 			temp.write(position, ptr + sizeof(Element), element->length);
 			position += element->length;
 		}
@@ -205,17 +204,19 @@ void MonitoringData::read(const char* user_name, TempSpace& temp)
 
 ULONG MonitoringData::setup(AttNumber att_id, const char* user_name)
 {
-	ensureSpace(sizeof(Element));
+	const ULONG offset = alignOffset(shared_memory->getHeader()->used);
+	const ULONG delta = offset + sizeof(Element) - shared_memory->getHeader()->used;
+
+	ensureSpace(delta);
 
 	// Prepare for writing new data at the tail
 
-	const ULONG offset = shared_memory->getHeader()->used;
 	UCHAR* const ptr = (UCHAR*) shared_memory->getHeader() + offset;
 	Element* const element = (Element*) ptr;
 	element->attId = att_id;
 	strncpy(element->userName, user_name, USERNAME_LENGTH);
 	element->length = 0;
-	shared_memory->getHeader()->used += alignOffset(sizeof(Element));
+	shared_memory->getHeader()->used += delta;
 	return offset;
 }
 
@@ -229,10 +230,8 @@ void MonitoringData::write(ULONG offset, ULONG length, const void* buffer)
 	UCHAR* const ptr = (UCHAR*) shared_memory->getHeader() + offset;
 	Element* const element = (Element*) ptr;
 	memcpy(ptr + sizeof(Element) + element->length, buffer, length);
-	ULONG previous = alignOffset(sizeof(Element) + element->length);
 	element->length += length;
-	ULONG current = alignOffset(sizeof(Element) + element->length);
-	shared_memory->getHeader()->used += (current - previous);
+	shared_memory->getHeader()->used += length;
 }
 
 
@@ -248,14 +247,20 @@ void MonitoringData::cleanup(AttNumber att_id)
 
 		if (element->attId == att_id)
 		{
-			fb_assert(shared_memory->getHeader()->used >= offset + length);
-			memmove(ptr, ptr + length, shared_memory->getHeader()->used - offset - length);
-			shared_memory->getHeader()->used -= length;
-		}
-		else
-		{
-			offset += length;
+			if (offset + length < shared_memory->getHeader()->used)
+			{
+				memmove(ptr, ptr + length, shared_memory->getHeader()->used - offset - length);
+				shared_memory->getHeader()->used -= length;
+			}
+			else
+			{
+				shared_memory->getHeader()->used = offset;
+			}
+
+			break;
 		}
+
+		offset += length;
 	}
 }
 
@@ -269,10 +274,11 @@ void MonitoringData::enumerate(SessionList& sessions, const char* user_name)
 		UCHAR* const ptr = (UCHAR*) shared_memory->getHeader() + offset;
 		const Element* const element = (Element*) ptr;
 		const ULONG length = alignOffset(sizeof(Element) + element->length);
-		offset += length;
 
 		if (!user_name || !strcmp(element->userName, user_name))
 			sessions.add(element->attId);
+
+		offset += length;
 	}
 }
 
@@ -1011,12 +1017,11 @@ void Monitoring::putRequest(SnapshotData::DumpRecord& record, const jrd_req* req
 	if (request->req_attachment)
 		record.storeInteger(f_mon_stmt_att_id, request->req_attachment->att_attachment_id);
 	// state, transaction ID, timestamp
-	if (request->req_flags & req_active)
+	if (request->req_transaction && (request->req_flags & req_active))
 	{
 		const bool is_stalled = (request->req_flags & req_stall);
 		record.storeInteger(f_mon_stmt_state, is_stalled ? mon_state_stalled : mon_state_active);
-		if (request->req_transaction)
-			record.storeInteger(f_mon_stmt_tra_id, request->req_transaction->tra_number);
+		record.storeInteger(f_mon_stmt_tra_id, request->req_transaction->tra_number);
 		record.storeTimestamp(f_mon_stmt_timestamp, request->req_timestamp);
 	}
 	else
@@ -1106,6 +1111,7 @@ void Monitoring::putCall(SnapshotData::DumpRecord& record, const jrd_req* reques
 	putMemoryUsage(record, request->req_memory_stats, stat_id, stat_call);
 }
 
+
 void Monitoring::putStatistics(SnapshotData::DumpRecord& record, const RuntimeStatistics& statistics,
 							   int stat_id, int stat_group)
 {
@@ -1176,6 +1182,7 @@ void Monitoring::putStatistics(SnapshotData::DumpRecord& record, const RuntimeSt
 	}
 }
 
+
 void Monitoring::putContextVars(SnapshotData::DumpRecord& record, const StringMap& variables,
 								SINT64 object_id, bool is_attachment)
 {
@@ -1195,6 +1202,7 @@ void Monitoring::putContextVars(SnapshotData::DumpRecord& record, const StringMa
 	}
 }
 
+
 void Monitoring::putMemoryUsage(SnapshotData::DumpRecord& record, const MemoryStats& stats,
 								int stat_id, int stat_group)
 {
@@ -1242,8 +1250,7 @@ void Monitoring::dumpAttachment(thread_db* tdbb, Attachment* attachment)
 	const AttNumber att_id = attachment->att_attachment_id;
 	const string& user_name = attachment->att_user->usr_user_name;
 
-	if (!dbb->dbb_monitoring_data)
-		dbb->dbb_monitoring_data = FB_NEW_POOL(pool) MonitoringData(dbb);
+	fb_assert(dbb->dbb_monitoring_data);
 
 	MonitoringData::Guard guard(dbb->dbb_monitoring_data);
 	dbb->dbb_monitoring_data->cleanup(att_id);
@@ -1306,15 +1313,15 @@ void Monitoring::publishAttachment(thread_db* tdbb)
 	Database* const dbb = tdbb->getDatabase();
 	Attachment* const attachment = tdbb->getAttachment();
 
-	if (!dbb->dbb_monitoring_data)
-		dbb->dbb_monitoring_data = FB_NEW_POOL(*dbb->dbb_permanent) MonitoringData(dbb);
-
 	const string& user_name = attachment->att_user->usr_user_name;
 
+	fb_assert(dbb->dbb_monitoring_data);
+
 	MonitoringData::Guard guard(dbb->dbb_monitoring_data);
 	dbb->dbb_monitoring_data->setup(attachment->att_attachment_id, user_name.c_str());
 }
 
+
 void Monitoring::cleanupAttachment(thread_db* tdbb)
 {
 	Database* const dbb = tdbb->getDatabase();
