@@ -371,7 +371,22 @@ void EXE_assignment(thread_db* tdbb, const ValueExprNode* to, dsc* from_desc, bo
 		{
 			// ASF: Don't let MOV_move call blb::move because MOV
 			// will not pass the destination field to blb::_move.
-			blb::move(tdbb, from_desc, to_desc, to);
+
+			record_param* rpb = NULL;
+			USHORT fieldId = 0;
+			if (to)
+			{
+				const FieldNode* toField = ExprNode::as<FieldNode>(to);
+				if (toField)
+				{
+					fieldId = toField->fieldId;
+					rpb = &request->req_rpb[toField->fieldStream];
+				}
+				else if (!(ExprNode::is<ParameterNode>(to) || ExprNode::is<VariableNode>(to)))
+					BUGCHECK(199);	// msg 199 expected field node
+			}
+
+			blb::move(tdbb, from_desc, to_desc, rpb, fieldId);
 		}
 		else if (!DSC_EQUIV(from_desc, to_desc, false))
 		{
@@ -667,6 +682,10 @@ void EXE_receive(thread_db* tdbb,
 						current->bli_request = NULL;
 					}
 				}
+				else
+				{
+					transaction->checkBlob(tdbb, id, false);
+				}
 			}
 		}
 	}
@@ -880,7 +899,7 @@ void EXE_unwind(thread_db* tdbb, jrd_req* request)
 	{
 		const JrdStatement* statement = request->getStatement();
 
-		if (statement->fors.getCount() || request->req_ext_stmt)
+		if (statement->fors.getCount() || request->req_ext_resultset || request->req_ext_stmt)
 		{
 			Jrd::ContextPoolHolder context(tdbb, request->req_pool);
 			jrd_req* old_request = tdbb->getRequest();
@@ -896,7 +915,10 @@ void EXE_unwind(thread_db* tdbb, jrd_req* request)
 				}
 
 				if (request->req_ext_resultset)
+				{
 					delete request->req_ext_resultset;
+					request->req_ext_resultset = NULL;
+				}
 
 				while (request->req_ext_stmt)
 					request->req_ext_stmt->close(tdbb);
