@@ -644,8 +644,8 @@ PAG PAG_allocate_pages(thread_db* tdbb, WIN* window, int cntAlloc, bool aligned)
 
 #ifdef VIO_DEBUG
 				VIO_trace(DEBUG_WRITES_INFO,
-					"\tPAG_allocate:  allocated page %" SLONGFORMAT"\n",
-							i + sequence * pageMgr.pagesPerPIP);
+					"PAG_allocate:  allocated page %" SLONGFORMAT"\n",
+					i + sequence * pageMgr.pagesPerPIP);
 #endif
 			}
 
@@ -715,8 +715,8 @@ PAG PAG_allocate_pages(thread_db* tdbb, WIN* window, int cntAlloc, bool aligned)
 
 #ifdef VIO_DEBUG
 				VIO_trace(DEBUG_WRITES_INFO,
-					"\tPAG_allocate:  allocated page %" SLONGFORMAT"\n",
-							bit + sequence * pageMgr.pagesPerPIP);
+					"PAG_allocate:  allocated page %" SLONGFORMAT"\n",
+					bit + sequence * pageMgr.pagesPerPIP);
 #endif
 			}
 
@@ -960,7 +960,20 @@ void PAG_format_header(thread_db* tdbb)
 	header->hdr_page_size = dbb->dbb_page_size;
 	header->hdr_ods_version = ODS_VERSION | ODS_FIREBIRD_FLAG;
 	DbImplementation::current.store(header);
-	header->hdr_ods_minor = ODS_CURRENT;
+
+#define ODS12_MINOR ODS_CURRENT
+
+#if defined(WIN_NT)
+#undef ODS12_MINOR
+#define ODS12_MINOR ODS_CURRENT12_0
+#endif // WIN_NT
+
+#if defined(LINUX) && SIZEOF_VOID_P == 8
+#undef ODS12_MINOR
+#define ODS12_MINOR ODS_CURRENT12_0
+#endif // LINUX-64bit
+
+	header->hdr_ods_minor = ODS12_MINOR;
 	header->hdr_oldest_transaction = 1;
 	header->hdr_end = HDR_SIZE;
 	header->hdr_data[0] = HDR_end;
@@ -970,6 +983,7 @@ void PAG_format_header(thread_db* tdbb)
 
 	dbb->dbb_ods_version = header->hdr_ods_version & ~ODS_FIREBIRD_FLAG;
 	dbb->dbb_minor_version = header->hdr_ods_minor;
+	dbb->dbb_implementation = DbImplementation(header);
 
 	CCH_RELEASE(tdbb, &window);
 }
@@ -1264,7 +1278,7 @@ void PAG_header_init(thread_db* tdbb)
 
 	const USHORT ods_version = header->hdr_ods_version & ~ODS_FIREBIRD_FLAG;
 
-	if (!Ods::isSupported(header->hdr_ods_version, header->hdr_ods_minor))
+	if (!Ods::isSupported(header))
 	{
 		ERR_post(Arg::Gds(isc_wrong_ods) << Arg::Str(attachment->att_filename) <<
 											Arg::Num(ods_version) <<
@@ -1295,6 +1309,7 @@ void PAG_header_init(thread_db* tdbb)
 
 	dbb->dbb_ods_version = ods_version;
 	dbb->dbb_minor_version = header->hdr_ods_minor;
+	dbb->dbb_implementation = DbImplementation(header);
 
 	dbb->dbb_page_size = header->hdr_page_size;
 	dbb->dbb_page_buffers = header->hdr_page_buffers;
@@ -1559,11 +1574,19 @@ void PAG_release_pages(thread_db* tdbb, USHORT pageSpaceID, int cntRelease,
 	page_inv_page* pages = NULL;
 	ULONG sequence = 0;
 
+#ifdef VIO_DEBUG
+	string dbg = "PAG_release_pages:  about to release pages: ";
+#endif
+
 	for (int i = 0; i < cntRelease; i++)
 	{
 #ifdef VIO_DEBUG
-		VIO_trace(DEBUG_WRITES_INFO,
-			"\tPAG_release_pages:  about to release page %" SLONGFORMAT"\n", pgNums[i]);
+		if (i > 0)
+			dbg.append(", ");
+
+		char num[16];
+		_ltoa_s(pgNums[i], num, sizeof(num), 10);
+		dbg.append(num);
 #endif
 
 		const ULONG seq = pgNums[i] / pageMgr.pagesPerPIP;
@@ -1598,6 +1621,10 @@ void PAG_release_pages(thread_db* tdbb, USHORT pageSpaceID, int cntRelease,
 		pages->pip_min = MIN(pages->pip_min, relative_bit);
 	}
 
+#ifdef VIO_DEBUG
+	VIO_trace(DEBUG_WRITES_INFO, "%s\n", dbg.c_str());
+#endif
+
 	pageSpace->pipHighWater.exchangeLower(sequence);
 
 	if (pages->pip_extent < pageMgr.pagesPerPIP)
@@ -1717,6 +1744,15 @@ void PAG_set_db_readonly(thread_db* tdbb, bool flag)
 		// for WRITE operations
 		header->hdr_flags &= ~hdr_read_only;
 		dbb->dbb_flags &= ~DBB_read_only;
+
+		// Also, reset in-memory Database transactions counters as it could be
+		// greater than the Next transaction stored on disk
+		dbb->dbb_oldest_active = Ods::getOAT(header);
+		dbb->dbb_oldest_snapshot = Ods::getOST(header);
+		dbb->dbb_oldest_transaction = Ods::getOIT(header);
+
+		fb_assert(dbb->dbb_next_transaction == Ods::getNT(header));
+		dbb->dbb_next_transaction = Ods::getNT(header);
 	}
 
 	CCH_MARK_MUST_WRITE(tdbb, &window);
@@ -2154,9 +2190,7 @@ ULONG PageSpace::lastUsedPage()
 	CCH_RELEASE(tdbb, &window);
 	pipMaxKnown = pipLast;
 
-	if (pipLast == pipFirst)
-		return last_bit + 1;
-	return last_bit + pipLast + 1;
+	return last_bit + (pipLast == pipFirst ? 0 : pipLast);
 }
 
 ULONG PageSpace::lastUsedPage(const Database* dbb)
