@@ -105,6 +105,7 @@ struct BlobIndex
 };
 
 typedef Firebird::BePlusTree<BlobIndex, ULONG, MemoryPool, BlobIndex> BlobIndexTree;
+typedef Firebird::BePlusTree<bid, bid, MemoryPool> FetchedBlobIdTree;
 
 // Transaction block
 
@@ -165,6 +166,7 @@ public:
 		tra_memory_stats(parent_stats),
 		tra_blobs_tree(p),
 		tra_blobs(outer ? outer->tra_blobs : &tra_blobs_tree),
+		tra_fetched_blobs(p),
 		tra_arrays(NULL),
 		tra_deferred_job(NULL),
 		tra_resources(*p),
@@ -239,7 +241,7 @@ public:
 		return tra_attachment->att_dsql_instance;
 	}
 
-	JTransaction* getInterface();
+	JTransaction* getInterface(bool create);
 	void setInterface(JTransaction* jt);
 
 	FB_API_HANDLE tra_public_handle;	// Public handle
@@ -250,13 +252,15 @@ public:
 	TraNumber tra_oldest_active;		// record versions older than this can be
 										// gargage-collected by this tx
 	TraNumber tra_att_oldest_active;	// oldest active transaction in the same attachment
-	jrd_tra*	tra_next;				// next transaction in attachment
+	jrd_tra* tra_next;					// next transaction in attachment
 	MemoryPool* const tra_pool;			// pool for transaction
 	Firebird::MemoryStats	tra_memory_stats;
 	BlobIndexTree tra_blobs_tree;		// list of active blobs
 	BlobIndexTree* tra_blobs;			// pointer to actual list of active blobs
+	FetchedBlobIdTree tra_fetched_blobs;	// list of fetched blobs
 	ArrayField*	tra_arrays;				// Linked list of active arrays
 	Lock*		tra_lock;				// lock for transaction
+	Lock*		tra_alter_db_lock;		// lock for ALTER DATABASE statement(s)
 	vec<Lock*>*			tra_relation_locks;	// locks for relations
 	TransactionBitmap*	tra_commit_sub_trans;	// committed sub-transactions
 	Savepoint*	tra_save_point;			// list of savepoints
@@ -358,12 +362,17 @@ public:
 		return record;
 	}
 
+	void unlinkFromAttachment();
+	void linkToAttachment(Attachment* attachment);
+	static void tra_abort(const char* reason);
+
 	UserManagement* getUserManagement();
 	SecDbContext* getSecDbContext();
 	SecDbContext* setSecDbContext(Firebird::IAttachment* att, Firebird::ITransaction* tra);
 	void eraseSecDbContext();
 	MappingList* getMappingList();
 	DbCreatorsList* getDbCreatorsList();
+	void checkBlob(thread_db* tdbb, const bid* blob_id, bool punt);
 
 	GenIdCache* getGenIdCache()
 	{
