@@ -54,7 +54,7 @@ set bulk_insert INSERT INTO SYSTEM_ERRORS (SQL_CODE, SQL_CLASS, SQL_SUBCLASS, FA
 (-817, 'HY', '000', 0, 51, 'segstr_no_write', 2, 'ERROR')
 (-901, 'HY', '000', 0, 52, 'segstr_wrong_db', 2, 'ERROR')
 (-902, 'HY', '000', 0, 53, 'sys_request', 4, 'SEVERE')
-(100, 'HY', '000', 0, 54, 'stream_eof', 2, 'ERROR')
+(-596, 'HY', '109', 0, 54, 'stream_eof', 2, 'ERROR')
 (-904, '08', '001', 0, 55, 'unavailable', NULL, 'ERROR')
 (-901, 'HY', '000', 0, 56, 'unres_rel', NULL, 'ERROR')
 (-901, 'HY', '000', 0, 57, 'uns_ext', NULL, 'SEVERE')
@@ -324,7 +324,7 @@ set bulk_insert INSERT INTO SYSTEM_ERRORS (SQL_CODE, SQL_CLASS, SQL_SUBCLASS, FA
 (-901, '42', '000', 0, 321, 'dsql_domain_not_found', NULL, NULL)
 (-284, '42', '000', 0, 322, 'index_unused', NULL, NULL)
 (-282, '42', '000', 0, 323, 'dsql_self_join', NULL, NULL)
-(-596, 'HY', '000', 0, 324, 'stream_bof', NULL, NULL)
+(-596, 'HY', '109', 0, 324, 'stream_bof', NULL, NULL)
 (-595, 'HY', '000', 0, 325, 'stream_crack', NULL, NULL)
 (-601, 'HY', '000', 0, 326, 'db_or_file_exists', NULL, NULL)
 (-401, '42', '000', 0, 327, 'invalid_operator', NULL, NULL)
@@ -673,7 +673,129 @@ set bulk_insert INSERT INTO SYSTEM_ERRORS (SQL_CODE, SQL_CLASS, SQL_SUBCLASS, FA
 (-901, 'HY', '000', 0, 665, 'out_of_temp_space', NULL, NULL)
 (-901, '42', '000', 0, 666, 'eds_expl_tran_ctrl', NULL, NULL)
 (-902, '28', '000', 0, 667, 'no_trusted_spb', NULL, NULL)
+(-901, '42', '000', 0, 668, 'package_name', NULL, NULL)
+(-901, '22', '006', 0, 669, 'cannot_make_not_null', NULL, NULL)
+(-901, '0A', '000', 0, 670, 'feature_removed', NULL, NULL)
+(-901, '00', '000', 0, 671, 'view_name', NULL, NULL)
+(-904, '08', '006', 0, 672, 'lock_dir_access', NULL, NULL)
+(-901, 'HY', '106', 0, 673, 'invalid_fetch_option', NULL, NULL)
+(-901, '2F', '000', 0, 674, 'bad_fun_BLR', NULL, NULL)
+(-901, '2F', '000', 0, 675, 'func_pack_not_implemented', NULL, NULL)
+(-901, '2F', '000', 0, 676, 'proc_pack_not_implemented', NULL, NULL)
+(-901, '39', '000', 0, 677, 'eem_func_not_returned', NULL, NULL)
+(-901, '39', '000', 0, 678, 'eem_proc_not_returned', NULL, NULL)
+(-901, '39', '000', 0, 679, 'eem_trig_not_returned', NULL, NULL)
+(-901, '39', '000', 0, 680, 'eem_bad_plugin_ver', NULL, NULL)
+(-901, '39', '000', 0, 681, 'eem_engine_notfound', NULL, NULL)
+(-532, '08', '002', 0, 682, 'attachment_in_use', NULL, NULL)
+(-532, '25', 'S02', 0, 683, 'transaction_in_use', NULL, NULL)
+(-901, '39', '000', 0, 684, 'pman_cannot_load_plugin', NULL, NULL)
+(-901, '00', '000', 0, 685, 'pman_module_notfound', NULL, NULL)
+(-901, '00', '000', 0, 686, 'pman_entrypoint_notfound', NULL, NULL)
+(-901, '00', '000', 0, 687, 'pman_module_bad', NULL, NULL)
+(-901, '00', '000', 0, 688, 'pman_plugin_notfound', NULL, NULL)
+(-833, '42', '000', 0, 689, 'sysf_invalid_trig_namespace', NULL, NULL)
+(-901, '22', '002', 0, 690, 'unexpected_null', NULL, NULL)
+(-901, '22', '00G', 0, 691, 'type_notcompat_blob', NULL, NULL)
+(-901, '22', '003', 0, 692, 'invalid_date_val', NULL, NULL)
+(-901, '22', '003', 0, 693, 'invalid_time_val', NULL, NULL)
+(-901, '22', '003', 0, 694, 'invalid_timestamp_val', NULL, NULL)
+(-901, '22', '003', 0, 695, 'invalid_index_val', NULL, NULL)
+(-836, '00', '000', 0, 696, 'formatted_exception', NULL, 'ERROR')
 (-532, 'HY', '018', 0, 697, 'async_active', NULL, NULL)
+(-901, '42', '000', 0, 698, 'private_function', NULL, NULL)
+(-901, '42', '000', 0, 699, 'private_procedure', NULL, NULL)
+(-904, '24', '000', 0, 700, 'request_outdated', NULL, NULL)
+(-901, '08', '003', 0, 701, 'bad_events_handle', NULL, 'ERROR')
+(-104, 'XX', '000', 0, 702, 'cannot_copy_stmt', NULL, NULL)
+(-104, '22', '000', 0, 703, 'invalid_boolean_usage', NULL, NULL)
+(-833, '42', '000', 0, 704, 'sysf_argscant_both_be_zero', NULL, NULL)
+(-901, 'HY', '000', 0, 705, 'spb_no_id', NULL, NULL)
+(-901, '42', '000', 0, 706, 'ee_blr_mismatch_null', NULL, NULL)
+(-901, '42', '000', 0, 707, 'ee_blr_mismatch_length', NULL, NULL)
+(-406, '42', '000', 0, 708, 'ss_out_of_bounds', NULL, NULL)
+(-902, '28', '000', 0, 709, 'missing_data_structures', NULL, NULL)
+(-902, '42', '000', 0, 710, 'protect_sys_tab', NULL, NULL)
+-- If you have better SQL code for this, change it.
+(-901, '2F', '000', 0, 711, 'libtommath_generic', NULL, NULL)
+(-902, 'HY', '000', 0, 712, 'wroblrver2', NULL, NULL)
+(-551, '28', '000', 0, 713, 'trunc_limits', NULL, NULL)
+(-551, '28', '000', 0, 714, 'info_access', NULL, NULL)
+(-104, 'HY', '024', 0, 715, 'svc_no_stdin', NULL, NULL)
+(-551, '28', '000', 0, 716, 'svc_start_failed', NULL, NULL)
+(-104, 'HY', '024', 0, 717, 'svc_no_switches', NULL, NULL)
+(-104, 'HY', '109', 0, 718, 'svc_bad_size', NULL, NULL)
+(-104, 'HY', '024', 0, 719, 'no_crypt_plugin', NULL, NULL)
+(-104, '22', '001', 0, 720, 'cp_name_too_long', NULL, NULL)
+(-901, '42', '818', 0, 721, 'cp_process_active', NULL, NULL)
+(-901, '42', '818', 0, 722, 'cp_already_crypted', NULL, NULL)
+(-902, 'XX', '000', 0, 723, 'decrypt_error', NULL, NULL)
+(-902, '39', '000', 0, 724, 'no_providers', NULL, NULL)
+(-104, '42', '818', 0, 725, 'null_spb', NULL, NULL)
+(-833, '42', '000', 0, 726, 'max_args_exceeded', NULL, NULL)
+(-901, '42', '000', 0, 727, 'ee_blr_mismatch_names_count', NULL, NULL)
+(-901, '42', '000', 0, 728, 'ee_blr_mismatch_name_not_found', NULL, NULL)
+(-901, '26', '000', 0, 729, 'bad_result_set', NULL, NULL)
+(-804, '07', '000', 0, 730, 'wrong_message_length', NULL, NULL)
+(-804, '07', '000', 0, 731, 'no_output_format', NULL, NULL)
+(-804, 'HY', '021', 0, 732, 'item_finish', NULL, NULL)
+(-902, 'XX', '000', 0, 733, 'miss_config', NULL, NULL)
+(-902, 'XX', '000', 0, 734, 'conf_line', NULL, NULL)
+(-902, 'XX', '000', 0, 735, 'conf_include', NULL, NULL)
+(-902, 'XX', '000', 0, 736, 'include_depth', NULL, NULL)
+(-902, 'XX', '000', 0, 737, 'include_miss', NULL, NULL)
+(-552, '28', '000', 0, 738, 'protect_ownership', NULL, NULL)
+(-901, 'HY', '000', 0, 739, 'badvarnum', NULL, NULL)
+(-902, '28', '000', 0, 740, 'sec_context', NULL, NULL);
+(-902, '28', '000', 0, 741, 'multi_segment', NULL, NULL);
+(-902, '28', '000', 0, 742, 'login_changed', NULL, NULL);
+(-902, '28', '000', 0, 743, 'auth_handshake_limit', NULL, NULL);
+(-902, '28', '000', 0, 744, 'wirecrypt_incompatible', NULL, NULL);
+(-902, '28', '000', 0, 745, 'miss_wirecrypt', NULL, NULL);
+(-902, '28', '000', 0, 746, 'wirecrypt_key', NULL, NULL);
+(-902, '28', '000', 0, 747, 'wirecrypt_plugin', NULL, NULL);
+(-902, '28', '000', 0, 748, 'secdb_name', NULL, NULL);
+(-902, '28', '000', 0, 749, 'auth_data', NULL, NULL);
+(-902, '28', '000', 0, 750, 'auth_datalength', NULL, NULL);
+(-901, 'HY', '007', 0, 751, 'info_unprepared_stmt', NULL, NULL);
+(-901, 'HY', '007', 0, 752, 'idx_key_value', NULL, NULL);
+(-901, 'HY', '000', 0, 753, 'forupdate_virtualtbl', NULL, NULL);
+(-901, 'HY', '000', 0, 754, 'forupdate_systbl', NULL, NULL);
+(-901, 'HY', '000', 0, 755, 'forupdate_temptbl', NULL, NULL);
+(-901, '42', '000', 0, 756, 'cant_modify_sysobj', NULL, NULL);
+(-901, '08', '004', 0, 757, 'server_misconfigured', NULL, NULL);
+(-901, '0A', '000', 0, 758, 'alter_role', NULL, NULL);
+(-901, '42', 'S01', 0, 759, 'map_already_exists', NULL, NULL);
+(-901, '42', 'S02', 0, 760, 'map_not_exists', NULL, NULL);
+(-901, '08', '004', 0, 761, 'map_load', NULL, NULL);
+(-901, '08', '004', 0, 762, 'map_aster', NULL, NULL);
+(-901, '08', '004', 0, 763, 'map_multi', NULL, NULL);
+(-901, '08', '004', 0, 764, 'map_undefined', NULL, NULL);
+(-924, 'HY', '000', 0, 765, 'baddpb_damaged_mode', NULL, NULL);
+(-924, 'HY', '000', 0, 766, 'baddpb_buffers_range', NULL, NULL);
+(-924, 'HY', '000', 0, 767, 'baddpb_temp_buffers', NULL, NULL);
+(-901, '0A', '000', 0, 768, 'map_nodb', NULL, NULL);
+(-901, '0A', '000', 0, 769, 'map_notable', NULL, NULL);
+(-901, '0P', '000', 0, 770, 'miss_trusted_role', NULL, NULL);
+(-901, '0P', '000', 0, 771, 'set_invalid_role', NULL, NULL);
+(-596, 'HY', '109', 0, 772, 'cursor_not_positioned', NULL, NULL)
+(-901, '42', '702', 0, 773, 'dup_attribute', NULL, NULL);
+(-901, '42', '000', 0, 774, 'dyn_no_priv', NULL, NULL)
+(-901, '42', '000', 0, 775, 'dsql_cant_grant_option', NULL, NULL);
+(-904, '40', '001', 0, 776, 'read_conflict', NULL, NULL);
+(-901, '08', '004', 0, 777, 'crdb_load', NULL, NULL);
+(-901, '0A', '000', 0, 778, 'crdb_nodb', NULL, NULL);
+(-901, '0A', '000', 0, 779, 'crdb_notable', NULL, NULL);
+(-804, 'HY', '000', 0, 780, 'interface_version_too_old', NULL, NULL)
+(-170, '07', '001', 0, 781, 'fun_param_mismatch', NULL, NULL)
+(-901, 'HY', '000', 0, 782, 'savepoint_backout_err', NULL, NULL)
+(-291, '42', '000', 0, 783, 'domain_primary_key_notnull', NULL, NULL)
+(-204, '2C', '000', 0, 784, 'invalid_attachment_charset', NULL, NULL)
+(-901, '08', '004', 0, 785, 'map_down', NULL, NULL)
+(-902, '08', '006', 0, 786, 'login_error', NULL, NULL)
+(-902, '08', '006', 0, 787, 'already_opened', NULL, NULL)
+(-902, '08', '006', 0, 788, 'bad_crypt_key', NULL, NULL)
+(-901, 'XX', '000', 0, 789, 'encrypt_error', NULL, NULL)
 -- GFIX
 (-901, '00', '000', 3, 1, 'gfix_db_name', NULL, NULL)
 (-901, '00', '000', 3, 2, 'gfix_invalid_sw', NULL, NULL)
@@ -708,7 +830,7 @@ set bulk_insert INSERT INTO SYSTEM_ERRORS (SQL_CODE, SQL_CLASS, SQL_SUBCLASS, FA
 (-104, 'HY', '000', 7, 3, 'dsql_transitional_numeric', NULL, NULL)
 (301, '01', '000', 7, 4, 'dsql_dialect_warning_expr', NULL, NULL)
 (-104, 'HY', '000', 7, 5, 'sql_db_dialect_dtype_unsupport', NULL, NULL)
-(-817, 'HY', '000', 7, 7, 'isc_sql_dialect_conflict_num', NULL, NULL)
+(-817, 'HY', '000', 7, 7, 'sql_dialect_conflict_num', NULL, NULL)
 (301, 'HY', '104', 7, 8, 'dsql_warning_number_ambiguous', NULL, NULL)
 (301, 'HY', '104', 7, 9, 'dsql_warning_number_ambiguous1', NULL, NULL)
 (301, 'HY', '104', 7, 10, 'dsql_warn_precision_ambiguous', NULL, NULL)
@@ -733,8 +855,30 @@ set bulk_insert INSERT INTO SYSTEM_ERRORS (SQL_CODE, SQL_CLASS, SQL_SUBCLASS, FA
 (-817, '54', '001', 7, 29, 'upd_ins_with_complex_view', NULL, NULL)
 (-817, '42', '000', 7, 30, 'dsql_incompatible_trigger_type', NULL, NULL)
 (-817, '42', '000', 7, 31, 'dsql_db_trigger_type_cant_change', NULL, NULL)
+(-607, 'HY', '000', 7, 32, 'dsql_record_version_table', NULL, NULL)
+(-802, '07', '002', 7, 33, 'dsql_invalid_sqlda_version', NULL, NULL)
+(-802, '07', '002', 7, 34, 'dsql_sqlvar_index', NULL, NULL)
+(-802, '07', '002', 7, 35, 'dsql_no_sqlind', NULL, NULL)
+(-802, '07', '002', 7, 36, 'dsql_no_sqldata', NULL, NULL)
+(-802, '07', '002', 7, 37, 'dsql_no_input_sqlda', NULL, NULL)
+(-802, '07', '002', 7, 38, 'dsql_no_output_sqlda', NULL, NULL)
+(-313, '07', '001', 7, 39, 'dsql_wrong_param_num', NULL, NULL)
 --  DYN
+(-901, '42', '000', 8, 37, 'dyn_filter_not_found', NULL, NULL)
+(-901, '42', '000', 8, 41, 'dyn_func_not_found', NULL, NULL)
+(-901, '42', '000', 8, 48, 'dyn_index_not_found', NULL, NULL)
+(-901, '42', '000', 8, 54, 'dyn_view_not_found', NULL, NULL)
+(-901, '42', '000', 8, 89, 'dyn_domain_not_found', NULL, NULL)
+(-901, '42', '000', 8, 109, 'dyn_cant_modify_auto_trig', NULL, NULL)
 (-901, '42', 'S01', 8, 132, 'dyn_dup_table', NULL, NULL)
+(-901, '42', '000', 8, 140, 'dyn_proc_not_found', NULL, NULL)
+(-901, '42', '000', 8, 144, 'dyn_exception_not_found', NULL, NULL)
+(-901, '42', '000', 8, 146, 'dyn_proc_param_not_found', NULL, NULL)
+(-901, '42', '000', 8, 147, 'dyn_trig_not_found', NULL, NULL)
+(-901, '42', '000', 8, 151, 'dyn_charset_not_found', NULL, NULL)
+(-901, '42', '000', 8, 152, 'dyn_collation_not_found', NULL, NULL)
+(-901, '42', '000', 8, 155, 'dyn_role_not_found', NULL, NULL)
+(-901, '42', '000', 8, 159, 'dyn_name_longer', NULL, NULL)
 (-901, '42', 'S22', 8, 176, 'dyn_column_does_not_exist', NULL, NULL)
 (-901, '28', '000', 8, 188, 'dyn_role_does_not_exist', NULL, NULL)
 (-901, '28', '000', 8, 189, 'dyn_no_grant_admin_opt', NULL, NULL)
@@ -753,9 +897,16 @@ set bulk_insert INSERT INTO SYSTEM_ERRORS (SQL_CODE, SQL_CLASS, SQL_SUBCLASS, FA
 (-829, '42', '000', 8, 209, 'dyn_invalid_dtype_conversion', NULL, NULL)
 (-829, '42', '000', 8, 210, 'dyn_dtype_conv_invalid', NULL, NULL)
 (-901, '42', '000', 8, 212, 'dyn_zero_len_id', NULL, NULL)
+(-901, '42', '000', 8, 214, 'dyn_gen_not_found', NULL, NULL)
 (-829, '2C', '000', 8, 221, 'max_coll_per_charset', NULL, NULL)
 (-829, 'HY', '000', 8, 222, 'invalid_coll_attr', NULL, NULL)
 (-901, 'HY', '000', 8, 232, 'dyn_wrong_gtt_scope', NULL, NULL)
+(-901, 'HY', '000', 8, 235, 'dyn_coll_used_table', NULL, NULL)
+(-901, 'HY', '000', 8, 236, 'dyn_coll_used_domain', NULL, NULL)
+(-607, 'HY', '000', 8, 237, 'dyn_cannot_del_syscoll', NULL, NULL)
+(-901, 'HY', '000', 8, 238, 'dyn_cannot_del_def_coll', NULL, NULL)
+(-901, '42', '000', 8, 241, 'dyn_table_not_found', NULL, NULL)
+(-901, 'HY', '000', 8, 243, 'dyn_coll_used_procedure', NULL, NULL)
 (-829, '42', '000', 8, 244, 'dyn_scale_too_big', NULL, NULL)
 (-829, '42', '000', 8, 245, 'dyn_precision_too_small', NULL, NULL)
 (106,  '42', '000', 8, 247, 'dyn_miss_priv_warning', NULL, NULL)
@@ -763,6 +914,32 @@ set bulk_insert INSERT INTO SYSTEM_ERRORS (SQL_CODE, SQL_CLASS, SQL_SUBCLASS, FA
 (-829, '42', '000', 8, 249, 'dyn_cannot_addrem_computed', NULL, NULL)
 (-901, '42', '000', 8, 250, 'dyn_no_empty_pw', NULL, NULL)
 (-901, '42', 'S11', 8, 251, 'dyn_dup_index', NULL, NULL)
+(-901, '42', '000', 8, 256, 'dyn_package_not_found', NULL, NULL)
+(-901, '42', '000', 8, 257, 'dyn_schema_not_found', NULL, NULL)
+(-607, 'HY', '000', 8, 258, 'dyn_cannot_mod_sysproc', NULL, NULL)
+(-607, 'HY', '000', 8, 259, 'dyn_cannot_mod_systrig', NULL, NULL)
+(-607, 'HY', '000', 8, 260, 'dyn_cannot_mod_sysfunc', NULL, NULL)
+(-607, 'HY', '000', 8, 261, 'dyn_invalid_ddl_proc', NULL, NULL)
+(-607, 'HY', '000', 8, 262, 'dyn_invalid_ddl_trig' NULL, NULL)
+(-901, '42', '000', 8, 263, 'dyn_funcnotdef_package', NULL, NULL)
+(-901, '42', '000', 8, 264, 'dyn_procnotdef_package', NULL, NULL)
+(-901, '42', '000', 8, 265, 'dyn_funcsignat_package', NULL, NULL)
+(-901, '42', '000', 8, 266, 'dyn_procsignat_package', NULL, NULL)
+(-901, '42', '000', 8, 267, 'dyn_defvaldecl_package_proc', NULL, NULL)
+(-901, '42', '000', 8, 269, 'dyn_package_body_exists', NULL, NULL)
+(-607, 'HY', '000', 8, 270, 'dyn_invalid_ddl_func' NULL, NULL)
+(-901, '42', '000', 8, 271, 'dyn_newfc_oldsyntax', NULL, NULL)
+(-901, '42', '000', 8, 278, 'dyn_func_param_not_found', NULL, NULL)
+(-901, '42', '000', 8, 279, 'dyn_routine_param_not_found', NULL, NULL)
+(-901, '42', '000', 8, 280, 'dyn_routine_param_ambiguous', NULL, NULL)
+(-901, 'HY', '000', 8, 281, 'dyn_coll_used_function', NULL, NULL)
+(-901, 'HY', '000', 8, 282, 'dyn_domain_used_function', NULL, NULL)
+(-901, '42', '000', 8, 283, 'dyn_alter_user_no_clause', NULL, NULL)
+(-901, '42', '000', 8, 286, 'dyn_duplicate_package_item', NULL, NULL)
+(-901, '42', '000', 8, 287, 'dyn_cant_modify_sysobj', NULL, NULL)
+(-901, '42', '000', 8, 288, 'dyn_cant_use_zero_increment', NULL, NULL)
+(-901, '42', '000', 8, 289, 'dyn_cant_use_in_foreignkey', NULL, NULL)
+(-901, '42', '000', 8, 290, 'dyn_defvaldecl_package_func', NULL, NULL)
 --  GBAK
 (-901, '00', '000', 12, 1, 'gbak_unknown_switch', NULL, NULL)
 (-901, '00', '000', 12, 2, 'gbak_page_size_missing', NULL, NULL)
@@ -849,8 +1026,20 @@ COMMIT WORK;
 (-901, '00', '000', 12, 279, 'gbak_mode_req', NULL, NULL)
 (-901, '00', '000', 12, 281, 'gbak_just_data', NULL, NULL)
 (-901, '00', '000', 12, 282, 'gbak_data_only', NULL, NULL)
+(-901, '00', '000', 12, 326, 'gbak_missing_interval', NULL, NULL)
+(-901, '00', '000', 12, 327, 'gbak_wrong_interval', NULL, NULL)
+(-901, '00', '000', 12, 329, 'gbak_verify_verbint', NULL, NULL)
+(-901, '00', '000', 12, 330, 'gbak_option_only_restore', NULL, NULL)
+(-901, '00', '000', 12, 331, 'gbak_option_only_backup', NULL, NULL)
+(-901, '00', '000', 12, 332, 'gbak_option_conflict', NULL, NULL)
+(-901, '00', '000', 12, 333, 'gbak_param_conflict', NULL, NULL)
+(-901, '00', '000', 12, 334, 'gbak_option_repeated', NULL, NULL)
+(-901, '00', '000', 12, 339, 'gbak_max_dbkey_recursion', NULL, NULL)
+(-901, '00', '000', 12, 340, 'gbak_max_dbkey_length', NULL, NULL)
 (-901, '00', '000', 12, 341, 'gbak_invalid_metadata', NULL, NULL)
 (-901, '00', '000', 12, 342, 'gbak_invalid_data', NULL, NULL)
+(-901, '00', '000', 12, 344, 'gbak_inv_bkup_ver2', NULL, NULL)
+(-901, '00', '000', 12, 348, 'gbak_db_format_too_old2', NULL, NULL)
 --  SQLERR
 (-804, 'HY', '000', 13, 917, 'dsql_too_old_ods', NULL, NULL)
 (-607, '42', 'S02', 13, 918, 'dsql_table_not_found', NULL, NULL)
@@ -907,6 +1096,82 @@ COMMIT WORK;
 (-833, '42', '000', 13, 968, 'dsql_invalid_type_neg', NULL, NULL)
 -- End of expression_eval_err subcodes
 (-104, '54', '011', 13, 969, 'dsql_max_distinct_items', NULL, NULL)
+(-901, '42', '000', 13, 970, 'dsql_alter_charset_failed', NULL, NULL);
+(-901, '42', '000', 13, 971, 'dsql_comment_on_failed', NULL, NULL);
+(-901, '42', '000', 13, 972, 'dsql_create_func_failed', NULL, NULL);
+(-901, '42', '000', 13, 973, 'dsql_alter_func_failed', NULL, NULL);
+(-901, '42', '000', 13, 974, 'dsql_create_alter_func_failed', NULL, NULL);
+(-901, '42', '000', 13, 975, 'dsql_drop_func_failed', NULL, NULL);
+(-901, '42', '000', 13, 976, 'dsql_recreate_func_failed', NULL, NULL);
+(-901, '42', '000', 13, 977, 'dsql_create_proc_failed', NULL, NULL);
+(-901, '42', '000', 13, 978, 'dsql_alter_proc_failed', NULL, NULL);
+(-901, '42', '000', 13, 979, 'dsql_create_alter_proc_failed', NULL, NULL);
+(-901, '42', '000', 13, 980, 'dsql_drop_proc_failed', NULL, NULL);
+(-901, '42', '000', 13, 981, 'dsql_recreate_proc_failed', NULL, NULL);
+(-901, '42', '000', 13, 982, 'dsql_create_trigger_failed', NULL, NULL);
+(-901, '42', '000', 13, 983, 'dsql_alter_trigger_failed', NULL, NULL);
+(-901, '42', '000', 13, 984, 'dsql_create_alter_trigger_failed', NULL, NULL);
+(-901, '42', '000', 13, 985, 'dsql_drop_trigger_failed', NULL, NULL);
+(-901, '42', '000', 13, 986, 'dsql_recreate_trigger_failed', NULL, NULL);
+(-901, '42', '000', 13, 987, 'dsql_create_collation_failed', NULL, NULL);
+(-901, '42', '000', 13, 988, 'dsql_drop_collation_failed', NULL, NULL);
+(-901, '42', '000', 13, 989, 'dsql_create_domain_failed', NULL, NULL);
+(-901, '42', '000', 13, 990, 'dsql_alter_domain_failed', NULL, NULL);
+(-901, '42', '000', 13, 991, 'dsql_drop_domain_failed', NULL, NULL);
+(-901, '42', '000', 13, 992, 'dsql_create_except_failed', NULL, NULL);
+(-901, '42', '000', 13, 993, 'dsql_alter_except_failed', NULL, NULL);
+(-901, '42', '000', 13, 994, 'dsql_create_alter_except_failed', NULL, NULL);
+(-901, '42', '000', 13, 995, 'dsql_recreate_except_failed', NULL, NULL);
+(-901, '42', '000', 13, 996, 'dsql_drop_except_failed', NULL, NULL);
+(-901, '42', '000', 13, 997, 'dsql_create_sequence_failed', NULL, NULL);
+(-901, '42', '000', 13, 998, 'dsql_create_table_failed', NULL, NULL);
+(-901, '42', '000', 13, 999, 'dsql_alter_table_failed', NULL, NULL);
+(-901, '42', '000', 13, 1000, 'dsql_drop_table_failed', NULL, NULL);
+(-901, '42', '000', 13, 1001, 'dsql_recreate_table_failed', NULL, NULL);
+(-901, '42', '000', 13, 1002, 'dsql_create_pack_failed', NULL, NULL);
+(-901, '42', '000', 13, 1003, 'dsql_alter_pack_failed', NULL, NULL);
+(-901, '42', '000', 13, 1004, 'dsql_create_alter_pack_failed', NULL, NULL);
+(-901, '42', '000', 13, 1005, 'dsql_drop_pack_failed', NULL, NULL);
+(-901, '42', '000', 13, 1006, 'dsql_recreate_pack_failed', NULL, NULL);
+(-901, '42', '000', 13, 1007, 'dsql_create_pack_body_failed', NULL, NULL);
+(-901, '42', '000', 13, 1008, 'dsql_drop_pack_body_failed', NULL, NULL);
+(-901, '42', '000', 13, 1009, 'dsql_recreate_pack_body_failed', NULL, NULL);
+(-901, '42', '000', 13, 1010, 'dsql_create_view_failed', NULL, NULL);
+(-901, '42', '000', 13, 1011, 'dsql_alter_view_failed', NULL, NULL);
+(-901, '42', '000', 13, 1012, 'dsql_create_alter_view_failed', NULL, NULL);
+(-901, '42', '000', 13, 1013, 'dsql_recreate_view_failed', NULL, NULL);
+(-901, '42', '000', 13, 1014, 'dsql_drop_view_failed', NULL, NULL);
+(-901, '42', '000', 13, 1015, 'dsql_drop_sequence_failed', NULL, NULL);
+(-901, '42', '000', 13, 1016, 'dsql_recreate_sequence_failed', NULL, NULL);
+(-901, '42', '000', 13, 1017, 'dsql_drop_index_failed', NULL, NULL);
+(-901, '42', '000', 13, 1018, 'dsql_drop_filter_failed', NULL, NULL);
+(-901, '42', '000', 13, 1019, 'dsql_drop_shadow_failed', NULL, NULL);
+(-901, '42', '000', 13, 1020, 'dsql_drop_role_failed', NULL, NULL);
+(-901, '42', '000', 13, 1021, 'dsql_drop_user_failed', NULL, NULL);
+(-901, '42', '000', 13, 1022, 'dsql_create_role_failed', NULL, NULL);
+(-901, '42', '000', 13, 1023, 'dsql_alter_role_failed', NULL, NULL);
+(-901, '42', '000', 13, 1024, 'dsql_alter_index_failed', NULL, NULL);
+(-901, '42', '000', 13, 1025, 'dsql_alter_database_failed', NULL, NULL);
+(-901, '42', '000', 13, 1026, 'dsql_create_shadow_failed', NULL, NULL);
+(-901, '42', '000', 13, 1027, 'dsql_create_filter_failed', NULL, NULL);
+(-901, '42', '000', 13, 1028, 'dsql_create_index_failed', NULL, NULL);
+(-901, '42', '000', 13, 1029, 'dsql_create_user_failed', NULL, NULL);
+(-901, '42', '000', 13, 1030, 'dsql_alter_user_failed', NULL, NULL);
+(-901, '42', '000', 13, 1031, 'dsql_grant_failed', NULL, NULL);
+(-901, '42', '000', 13, 1032, 'dsql_revoke_failed', NULL, NULL);
+(-104, '42', '000', 13, 1033, 'dsql_cte_recursive_aggregate', NULL, NULL)
+(-901, '42', '000', 13, 1034, 'dsql_mapping_failed', NULL, NULL)
+(-901, '42', '000', 13, 1035, 'dsql_alter_sequence_failed', NULL, NULL)
+(-901, '42', '000', 13, 1036, 'dsql_create_generator_failed', NULL, NULL)
+(-901, '42', '000', 13, 1037, 'dsql_set_generator_failed', NULL, NULL)
+(-104, '42', '000', 13, 1038, 'dsql_wlock_simple', NULL, NULL)
+(-104, '42', '000', 13, 1039, 'dsql_firstskip_rows', NULL, NULL)
+(-104, '42', '000', 13, 1040, 'dsql_wlock_aggregates', NULL, NULL)
+(-104, '42', '000', 13, 1041, 'dsql_wlock_conflict', NULL, NULL)
+(-901, '07', '002', 13, 1042, 'dsql_max_exception_arguments', NULL, NULL)
+(-901, '42', '000', 13, 1043, 'dsql_string_byte_length', NULL, NULL)
+(-901, '42', '000', 13, 1044, 'dsql_string_char_length', NULL, NULL)
+(-901, '07', '002', 13, 1045, 'dsql_max_nesting', NULL, NULL)
 -- GSEC
 (-901, '00', '000', 18, 15, 'gsec_cant_open_db', NULL, NULL)
 (-901, '00', '000', 18, 16, 'gsec_switches_error', NULL, NULL)
@@ -940,27 +1205,6 @@ COMMIT WORK;
 (-901, '00', '000', 18, 79, 'gsec_db_admin_specified', NULL, NULL)
 (-901, '00', '000', 18, 80, 'gsec_db_admin_pw_specified', NULL, NULL)
 (-901, '00', '000', 18, 81, 'gsec_sql_role_specified', NULL, NULL)
--- LICENSE
-(-901, '00', '000', 19, 0, 'license_no_file', NULL, NULL)
-(-901, '00', '000', 19, 19, 'license_op_specified', NULL, NULL)
-(-901, '00', '000', 19, 20, 'license_op_missing', NULL, NULL)
-(-901, '00', '000', 19, 21, 'license_inv_switch', NULL, NULL)
-(-901, '00', '000', 19, 22, 'license_inv_switch_combo', NULL, NULL)
-(-901, '00', '000', 19, 23, 'license_inv_op_combo', NULL, NULL)
-(-901, '00', '000', 19, 24, 'license_amb_switch', NULL, NULL)
-(-901, '00', '000', 19, 25, 'license_inv_parameter', NULL, NULL)
-(-901, '00', '000', 19, 26, 'license_param_specified', NULL, NULL)
-(-901, '00', '000', 19, 27, 'license_param_req', NULL, NULL)
-(-901, '00', '000', 19, 28, 'license_syntx_error', NULL, NULL)
-(-901, '00', '000', 19, 30, 'license_dup_id', NULL, NULL)
-(-901, '00', '000', 19, 31, 'license_inv_id_key', NULL, NULL)
-(-901, '00', '000', 19, 32, 'license_err_remove', NULL, NULL)
-(-901, '00', '000', 19, 33, 'license_err_update', NULL, NULL)
-(-901, '00', '000', 19, 34, 'license_err_convert', NULL, NULL)
-(-901, '00', '000', 19, 35, 'license_err_unk', NULL, NULL)
-(-901, '00', '000', 19, 36, 'license_svc_err_add', NULL, NULL)
-(-901, '00', '000', 19, 37, 'license_svc_err_remove', NULL, NULL)
-(-901, '00', '000', 19, 59, 'license_eval_exists', NULL, NULL)
 -- GSTAT
 (-901, '00', '000', 21, 1, 'gstat_unknown_switch', NULL, NULL)
 (-901, '00', '000', 21, 2, 'gstat_retry', NULL, NULL)
@@ -983,8 +1227,59 @@ COMMIT WORK;
 (-901, '00', '000', 22, 52, 'fbsvcmgr_bad_arg', NULL, NULL)
 -- UTILITIES
 (-901, '00', '000', 23, 1, 'utl_trusted_switch', NULL, NULL)
---
+-- NBACKUP
+(-901, '00', '000', 24, 29, 'nbackup_missing_param', NULL, NULL)
+(-901, '00', '000', 24, 30, 'nbackup_allowed_switches', NULL, NULL)
+(-901, '00', '000', 24, 31, 'nbackup_unknown_param', NULL, NULL)
+(-901, '00', '000', 24, 32, 'nbackup_unknown_switch', NULL, NULL)
+(-901, '00', '000', 24, 33, 'nbackup_nofetchpw_svc', NULL, NULL)
+(-901, '00', '000', 24, 34, 'nbackup_pwfile_error', NULL, NULL)
+(-901, '00', '000', 24, 35, 'nbackup_size_with_lock', NULL, NULL)
+(-901, '00', '000', 24, 36, 'nbackup_no_switch', NULL, NULL)
+(-901, '00', '000', 24, 39, 'nbackup_err_read', NULL, NULL)
+(-901, '00', '000', 24, 40, 'nbackup_err_write', NULL, NULL)
+(-901, '00', '000', 24, 41, 'nbackup_err_seek', NULL, NULL)
+(-901, '00', '000', 24, 42, 'nbackup_err_opendb', NULL, NULL)
+(-901, '00', '000', 24, 43, 'nbackup_err_fadvice', NULL, NULL)
+(-901, '00', '000', 24, 44, 'nbackup_err_createdb', NULL, NULL)
+(-901, '00', '000', 24, 45, 'nbackup_err_openbk', NULL, NULL)
+(-901, '00', '000', 24, 46, 'nbackup_err_createbk', NULL, NULL)
+(-901, '00', '000', 24, 47, 'nbackup_err_eofdb', NULL, NULL)
+(-901, '00', '000', 24, 48, 'nbackup_fixup_wrongstate', NULL, NULL)
+(-901, '00', '000', 24, 49, 'nbackup_err_db', NULL, NULL)
+(-901, '00', '000', 24, 50, 'nbackup_userpw_toolong', NULL, NULL)
+(-901, '00', '000', 24, 51, 'nbackup_lostrec_db', NULL, NULL)
+(-901, '00', '000', 24, 52, 'nbackup_lostguid_db', NULL, NULL)
+(-901, '00', '000', 24, 53, 'nbackup_err_eofhdrdb', NULL, NULL)
+(-901, '00', '000', 24, 54, 'nbackup_db_notlock', NULL, NULL)
+(-901, '00', '000', 24, 55, 'nbackup_lostguid_bk', NULL, NULL)
+(-901, '00', '000', 24, 56, 'nbackup_page_changed', NULL, NULL)
+(-901, '00', '000', 24, 57, 'nbackup_dbsize_inconsistent', NULL, NULL)
+(-901, '00', '000', 24, 58, 'nbackup_failed_lzbk', NULL, NULL)
+(-901, '00', '000', 24, 59, 'nbackup_err_eofhdrbk', NULL, NULL)
+(-901, '00', '000', 24, 60, 'nbackup_invalid_incbk', NULL, NULL)
+(-901, '00', '000', 24, 61, 'nbackup_unsupvers_incbk', NULL, NULL)
+(-901, '00', '000', 24, 62, 'nbackup_invlevel_incbk', NULL, NULL)
+(-901, '00', '000', 24, 63, 'nbackup_wrong_orderbk', NULL, NULL)
+(-901, '00', '000', 24, 64, 'nbackup_err_eofbk', NULL, NULL)
+(-901, '00', '000', 24, 65, 'nbackup_err_copy', NULL, NULL)
+(-901, '00', '000', 24, 66, 'nbackup_err_eofhdr_restdb', NULL, NULL)
+(-901, '00', '000', 24, 67, 'nbackup_lostguid_l0bk', NULL, NULL)
+(-901, '00', '000', 24, 71, 'nbackup_switchd_parameter', NULL, NULL)
+(-901, '08', '006', 24, 73, 'nbackup_user_stop', NULL, NULL)
+(-901, '54', '023', 24, 75, 'nbackup_deco_parse', NULL, NULL)
+-- FBTRACEMGR
+(-901, '00', '000', 25, 30, 'trace_conflict_acts', NULL, NULL)
+(-901, '00', '000', 25, 31, 'trace_act_notfound', NULL, NULL)
+(-901, '00', '000', 25, 32, 'trace_switch_once', NULL, NULL)
+(-901, '00', '000', 25, 33, 'trace_param_val_miss', NULL, NULL)
+(-901, '00', '000', 25, 34, 'trace_param_invalid', NULL, NULL)
+(-901, '00', '000', 25, 35, 'trace_switch_unknown', NULL, NULL)
+(-901, '00', '000', 25, 36, 'trace_switch_svc_only', NULL, NULL)
+(-901, '00', '000', 25, 37, 'trace_switch_user_only', NULL, NULL)
+(-901, '00', '000', 25, 38, 'trace_switch_param_miss', NULL, NULL)
+(-901, '00', '000', 25, 39, 'trace_param_act_notcompat', NULL, NULL)
+(-901, '00', '000', 25, 40, 'trace_mandatory_switch_miss' NULL, NULL)
 stop
 
 COMMIT WORK;
-
