@@ -675,7 +675,15 @@ void NBackup::create_backup()
 	}
 	else
 	{
-		backup = CreateFile(nm.c_str(), GENERIC_WRITE, FILE_SHARE_DELETE,
+		// See CORE-4913 and "Create File" article on MSDN:
+		// When an application creates a file across a network, it is better to use 
+		// GENERIC_READ | GENERIC_WRITE for dwDesiredAccess than to use GENERIC_WRITE 
+		// alone. The resulting code is faster, because the redirector can use the 
+		// cache manager and send fewer SMBs with more data. This combination also 
+		// avoids an issue where writing to a file across a network can occasionally 
+		// return ERROR_ACCESS_DENIED.
+
+		backup = CreateFile(nm.c_str(), GENERIC_READ | GENERIC_WRITE, FILE_SHARE_DELETE,
 			NULL, CREATE_NEW, FILE_ATTRIBUTE_NORMAL | FILE_FLAG_SEQUENTIAL_SCAN, NULL);
 	}
 	if (backup != INVALID_HANDLE_VALUE)
@@ -1043,7 +1051,7 @@ void NBackup::backup_database(int level, const PathName& fname)
 		if (read_file(dbase, header, SECTOR_ALIGNMENT/*sizeof(*header)*/) != SECTOR_ALIGNMENT/*sizeof(*header)*/)
 			status_exception::raise(Arg::Gds(isc_nbackup_err_eofhdrdb) << dbname.c_str() << Arg::Num(1));
 
-		if (!Ods::isSupported(header->hdr_ods_version, header->hdr_ods_minor))
+		if (!Ods::isSupported(header))
 		{
 			const USHORT ods_version = header->hdr_ods_version & ~ODS_FIREBIRD_FLAG;
 			status_exception::raise(Arg::Gds(isc_wrong_ods) << Arg::Str(database.c_str()) <<
@@ -1430,7 +1438,18 @@ void NBackup::restore_database(const BackupFiles& files)
 				// We may also add SCN check, but GUID check covers this case too
 				if (memcmp(&bakheader.prev_guid, &prev_guid, sizeof(Guid)) != 0)
 					status_exception::raise(Arg::Gds(isc_nbackup_wrong_orderbk) << bakname.c_str());
-				seek_file(backup, bakheader.page_size);
+
+				// Emulate seek_file(backup, bakheader.page_size)
+				// Backup is stream-oriented, if -decompress is used pipe can't be seek()'ed
+				FB_SIZE_T left = bakheader.page_size - sizeof(bakheader);
+				while (left)
+				{
+					char char_buf[1024];
+					FB_SIZE_T step = left > sizeof(char_buf) ? sizeof(char_buf) : left;
+					if (read_file(backup, &char_buf, step) != step)
+						status_exception::raise(Arg::Gds(isc_nbackup_err_eofhdrbk) << bakname.c_str());
+					left -= step;
+				}
 
 				delete_database = true;
 				prev_guid = bakheader.backup_guid;
