@@ -29,7 +29,7 @@
 #include "TraceUnicodeUtils.h"
 #include "../../jrd/evl_string.h"
 #include "../../jrd/SimilarToMatcher.h"
-#include "../../jrd/isc_f_proto.h"
+#include "../../common/isc_f_proto.h"
 
 using namespace Firebird;
 
@@ -45,14 +45,14 @@ void TraceCfgReader::readTraceConfiguration(const char* text,
 
 #define PATH_PARAMETER(NAME, VALUE) \
 	if (!found && el->name == #NAME) { \
-		string temp; \
+		Firebird::PathName temp; \
 		expandPattern(el, temp); \
 		m_config.NAME = temp.c_str(); \
 		found = true; \
 	}
 #define STR_PARAMETER(NAME, VALUE) \
 	if (!found && el->name == #NAME) { \
-		m_config.NAME = el->getAttributeName(0); \
+		m_config.NAME = el->value; \
 		found = true; \
 	}
 #define BOOL_PARAMETER(NAME, VALUE) \
@@ -69,7 +69,7 @@ void TraceCfgReader::readTraceConfiguration(const char* text,
 
 namespace
 {
-	template <typename PrevConverter>
+	template <typename PrevConverter = Jrd::NullStrConverter>
 	class SystemToUtf8Converter : public PrevConverter
 	{
 	public:
@@ -91,21 +91,7 @@ namespace
 
 void TraceCfgReader::readConfig()
 {
-	Firebird::AutoPtr<ConfigFile> cfgFile(new ConfigFile(Lex::LEX_none));
-	cfgFile->addText(m_text);
-	try
-	{
-		cfgFile->parse();
-	}
-	catch (const AdminException& ex)
-	{
-		fatal_exception::raiseFmt(ERROR_PREFIX"%s",
-			ex.getText());
-	}
-	catch (...)
-	{
-		fatal_exception::raiseFmt("unknown error while parsing trace configuration");
-	}
+	ConfigFile cfgFile(ConfigFile::USE_TEXT, m_text, ConfigFile::HAS_SUB_CONF | ConfigFile::NATIVE_ORDER);
 
 	m_subpatterns[0].start = 0;
 	m_subpatterns[0].end = m_databaseName.length();
@@ -116,14 +102,16 @@ void TraceCfgReader::readConfig()
 	}
 
 	bool defDB = false, defSvc = false, exactMatch = false;
-	const Element* section = cfgFile->getObjects()->children;
-	for (; section && !exactMatch; section = section->sibling)
+	const ConfigFile::Parameters& params = cfgFile.getParameters();
+	for (FB_SIZE_T n = 0; n < params.getCount() && !exactMatch; ++n)
 	{
+		const ConfigFile::Parameter* section = &params[n];
+
 		const bool isDatabase = (section->name == "database");
 		if (!isDatabase && section->name != "services")
 			continue;
 
-		const string pattern = section->getAttributes() ? section->getAttributeName(0) : "";
+		const ConfigFile::String pattern = section->value;
 		bool match = false;
 		if (pattern.empty())
 		{
@@ -133,10 +121,11 @@ void TraceCfgReader::readConfig()
 				{
 					fatal_exception::raiseFmt(ERROR_PREFIX
 						"line %d: second default database section is not allowed",
-						section->lineNumber + 1);
+						section->line);
 				}
 
 				match = !m_databaseName.empty();
+				//match = m_databaseName.empty();
 				defDB = true;
 			}
 			else
@@ -145,7 +134,7 @@ void TraceCfgReader::readConfig()
 				{
 					fatal_exception::raiseFmt(ERROR_PREFIX
 						"line %d: second default service section is not allowed",
-						section->lineNumber + 1);
+						section->line);
 				}
 				match = m_databaseName.empty();
 				defSvc = true;
@@ -153,7 +142,10 @@ void TraceCfgReader::readConfig()
 		}
 		else if (isDatabase && !m_databaseName.empty())
 		{
-			if (m_databaseName == pattern.c_str())
+			PathName noQuotePattern = pattern.ToPathName();
+			noQuotePattern.alltrim(" '\'");
+
+			if (m_databaseName == noQuotePattern)
 				match = exactMatch = true;
 			else
 			{
@@ -161,16 +153,15 @@ void TraceCfgReader::readConfig()
 				try
 				{
 #ifdef WIN_NT	// !CASE_SENSITIVITY
-					typedef Jrd::UpcaseConverter<SystemToUtf8Converter<Jrd::NullStrConverter> >
-						SimilarConverter;
+					typedef Jrd::UpcaseConverter<SystemToUtf8Converter<> > SimilarConverter;
 #else
-					typedef SystemToUtf8Converter<Jrd::NullStrConverter> SimilarConverter;
+					typedef SystemToUtf8Converter<> SimilarConverter;
 #endif
 
 					UnicodeCollationHolder unicodeCollation(*getDefaultMemoryPool());
-					Jrd::TextType *textType = unicodeCollation.getTextType();
+					Jrd::TextType* textType = unicodeCollation.getTextType();
 
-					SimilarToMatcher<Jrd::CanonicalConverter<SimilarConverter>, ULONG> matcher(
+					SimilarToMatcher<ULONG, Jrd::CanonicalConverter<SimilarConverter> > matcher(
 						*getDefaultMemoryPool(), textType, (const UCHAR*) pattern.c_str(),
 						pattern.length(), '\\', true);
 
@@ -194,15 +185,17 @@ void TraceCfgReader::readConfig()
 				}
 				catch (const Exception&)
 				{
-					if (regExpOk) {
+					if (regExpOk)
+					{
 						fatal_exception::raiseFmt(ERROR_PREFIX
 							"line %d: error while processing string \"%s\" against regular expression \"%s\"",
-							section->lineNumber + 1, m_databaseName.c_str(), pattern.c_str());
+							section->line, m_databaseName.c_str(), pattern.c_str());
 					}
-					else {
+					else
+					{
 						fatal_exception::raiseFmt(ERROR_PREFIX
 							"line %d: error while compiling regular expression \"%s\"",
-							section->lineNumber + 1, pattern.c_str());
+							section->line, pattern.c_str());
 					}
 				}
 			}
@@ -211,13 +204,22 @@ void TraceCfgReader::readConfig()
 		if (!match)
 			continue;
 
-		for (const Element* el = section->children; el; el = el->sibling)
+		if (!section->sub)
 		{
-			if (!el->getAttributes())
+			fatal_exception::raiseFmt(ERROR_PREFIX
+				"Trace parameters are not present");
+		}
+
+		const ConfigFile::Parameters& elements = section->sub->getParameters();
+		for (FB_SIZE_T p = 0; p < elements.getCount(); ++p)
+		{
+			const ConfigFile::Parameter* el = &elements[p];
+
+			if (!el->value.hasData())
 			{
 				fatal_exception::raiseFmt(ERROR_PREFIX
 					"line %d: element \"%s\" have no attribute value set",
-					el->lineNumber + 1, el->name.c_str());
+					el->line, el->name.c_str());
 			}
 
 			bool found = false;
@@ -238,7 +240,7 @@ void TraceCfgReader::readConfig()
 			{
 				fatal_exception::raiseFmt(ERROR_PREFIX
 					"line %d: element \"%s\" is unknown",
-					el->lineNumber + 1, el->name.c_str());
+					el->line, el->name.c_str());
 			}
 		}
 	}
@@ -249,10 +251,9 @@ void TraceCfgReader::readConfig()
 #undef BOOL_PARAMETER
 #undef UINT_PARAMETER
 
-bool TraceCfgReader::parseBoolean(const Element* el) const
+bool TraceCfgReader::parseBoolean(const ConfigFile::Parameter* el) const
 {
-	const char* value = el->getAttributeName(0);
-	string tempValue(value);
+	ConfigFile::String tempValue(el->value);
 	tempValue.upper();
 
 	if (tempValue == "1" || tempValue == "ON" || tempValue == "YES" || tempValue == "TRUE")
@@ -262,35 +263,42 @@ bool TraceCfgReader::parseBoolean(const Element* el) const
 
 	fatal_exception::raiseFmt(ERROR_PREFIX
 		"line %d, element \"%s\": \"%s\" is not a valid boolean value",
-		el->lineNumber + 1, el->name.c_str(), value);
+		el->line, el->name.c_str(), el->value.c_str());
 	return false; // Silence the compiler
 }
 
-ULONG TraceCfgReader::parseUInteger(const Element* el) const
+ULONG TraceCfgReader::parseUInteger(const ConfigFile::Parameter* el) const
 {
-	const char *value = el->getAttributeName(0);
+	const char *value = el->value.c_str();
 	ULONG result = 0;
-	if (!sscanf(value, "%" ULONGFORMAT, &result)) {
+	if (!sscanf(value, "%" ULONGFORMAT, &result))
+	{
 		fatal_exception::raiseFmt(ERROR_PREFIX
 			"line %d, element \"%s\": \"%s\" is not a valid integer value",
-			el->lineNumber + 1, el->name.c_str(), value);
+			el->line, el->name.c_str(), value);
 	}
 	return result;
 }
 
-void TraceCfgReader::expandPattern(const Element* el, string& valueToExpand)
+void TraceCfgReader::expandPattern(const ConfigFile::Parameter* el, PathName& valueToExpand)
 {
-	valueToExpand = el->getAttributeName(0);
-	string::size_type pos = 0;
+	valueToExpand = el->value.ToPathName();
+
+	// strip quotes around value, if any
+	valueToExpand.alltrim(" '\"");
+
+	PathName::size_type pos = 0;
 	while (pos < valueToExpand.length())
 	{
 		string::char_type c = valueToExpand[pos];
 		if (c == '\\')
 		{
 			if (pos + 1 >= valueToExpand.length())
+			{
 				fatal_exception::raiseFmt(ERROR_PREFIX
 					"line %d, element \"%s\": pattern is invalid\n\t %s",
-					el->lineNumber + 1, el->name.c_str(), el->getAttributeName(0));
+					el->line, el->name.c_str(), el->value.c_str());
+			}
 
 			c = valueToExpand[pos + 1];
 			if (c == '\\')
@@ -319,7 +327,7 @@ void TraceCfgReader::expandPattern(const Element* el, string& valueToExpand)
 
 			fatal_exception::raiseFmt(ERROR_PREFIX
 				"line %d, element \"%s\": pattern is invalid\n\t %s",
-				el->lineNumber + 1, el->name.c_str(), el->getAttributeName(0));
+				el->line, el->name.c_str(), el->value.c_str());
 		}
 
 		pos++;
