@@ -216,9 +216,10 @@ _br_find_exe_for_symbol (const void *symbol, BrInitError *error)
 			break;
 
 		/* Sanity check. */
-		if (strstr (line, " r-xp ") == NULL || strchr (line, '/') == NULL)
+		if ( !(strstr(line, " r-xp ") || strstr(line, " r--p "))
+			 || strchr (line, '/') == NULL)
 			continue;
-
+		
 		/* Parse line. */
 		start_addr = line;
 		end_addr = strchr (line, '-');
